# ===============================================================
# TestMatrixOperations.py: 
# 
# ===============================================================

from numpy import array
from numpy.linalg import eig

from scipy.linalg import lu
from scipy.linalg import qr

# main method ...

def main():
    print("--- Enter TestMatrixOperations.main()     ... ");
    print("--- ======================================== ... ");

    # Define square 3x3 matrix ...

    print("--- Define square 3x3 matrix ...");

    A = array([ [1,2,3],
                [4,5,6],
                [7,8,9]
              ])
    print(A)
    print(A.shape)

    # Compute eigenvalues and eigenvectors ...

    print("--- Compute eigenvalues and eigenvectors ...");

    evals, evects = eig(A)

    print(evals)
    print(evects)
    print(evals.shape)
    print(evects.shape)

    # Factorize A ...

    print("--- Factorize A --> P, L, U ...");

    P, L, U = lu(A)

    print(P)
    print(L)
    print(U)
    print(A.shape)
    print(P.shape)
    print(L.shape)
    print(U.shape)

    # Reconstruct A from P.L.U

    print("--- Reconstruct A from P.L.U ...");

    B = P.dot(L).dot(U)

    print(B)
    print(B.shape)

    # Define square matrix

    print("--- Define rectangular matrix A ...");

    A = array([ [1,2], [4,5], [7,8] ])

    print(A)

    # Factorize

    print("--- Compute QR decomposition of A ...");

    Q, R = qr(A, mode='full')

    print(Q)
    print(R)
    print(A.shape)
    print(Q.shape)
    print(R.shape)

    print("--- Reconstruct A from Q.R ...");

    B = Q.dot(R)
    print(B)
    print(B.shape)

    print("--- ======================================== ... ");
    print("--- Leave TestMatrixOperations.main()        ... ");

# call the main method ...

main()
