# ===============================================================
# TestNumpySVD.py: Exercise SVD (singular-value decomposition)
# routines in Python. 
# 
# Focus: Evaluation of eigenvalues, eigenvectors for matrix 
# adjaceny matrices. 
# 
# Written by: Mark Austin                            October 2020 
# ===============================================================

import numpy as np  # Make numpy available using np.
import numpy.matlib

from numpy.linalg import matrix_rank
from numpy import linalg as LA

# Function to print vectors ...

def PrintVector(a):
    for col in a:
       print("{:8.2f}".format(col), end=" ")
    print("")

# Function to print matrices ...

def PrintMatrix(a):
    for row in a:
        for col in row:
            print("{:8.2f}".format(col), end=" ")
        print("")

# main method ...

def main():
    print("--- Enter TestNumpySVD.main()                ... ");
    print("--- ======================================== ... ");

    # Part 1: Adjacency for six node ring ...
    # ---------------------------------------

    print("--- ");
    print("--- Part 1: Adjacency matrix for six node ring ... ");
    print("--- ------------------------------------------ ... ");
    print("--- Adjacency matrix Adj01 ... ");

    Adj01 = [ [ 0, 1, 0, 0, 0, 1 ],
              [ 1, 0, 1, 0, 0, 0 ],
              [ 0, 1, 0, 1, 0, 0 ],
              [ 0, 0, 1, 0, 1, 0 ],
              [ 0, 0, 0, 1, 0, 1 ],
              [ 1, 0, 0, 0, 1, 0 ] ];

    PrintMatrix(Adj01)

    # Compute rank of adj01 ...

    adj01rank = matrix_rank(Adj01)
    print("--- Rank(Adj01) = {:6,.1f}".format( adj01rank ));

    # Compute rank of adj01 + I ...

    adj01I = numpy.matlib.identity(6,dtype=int);

    print(Adj01 + adj01I)

    adj01rank = matrix_rank(Adj01 + adj01I )
    print("--- Rank(A+I) = {:6,.1f}".format( adj01rank ));

    # Compute eigenvalues of A ...

    w, v = LA.eig( Adj01 )

    print("--- Eigenvalues (A) ...");
    PrintVector(w)
    print("--- Eigenvectors (A) ...");
    PrintMatrix(v)

    # Compute eigenvalues of A + adj01I ...

    temp = Adj01 + adj01I;
    w, v = LA.eig( temp )

    print("--- Eigenvalues (A+I) ...");
    PrintVector(w)
    print("--- Eigenvectors (A+I) ...");
    print(v)

    # Part 2: Fully connected six node graph ...
    # -------------------------------------------

    print("--- ");
    print("--- Part 2: Adjacency matrix for six node fully connected graph ... ");
    print("--- ----------------------------------------------------------- ... ");
    print("--- Adjacency matrix Adj02 ... ");

    Adj02 = [ [ 0, 1, 1, 1, 1, 1 ],
              [ 1, 0, 1, 1, 1, 1 ],
              [ 1, 1, 0, 1, 1, 1 ],
              [ 1, 1, 1, 0, 1, 1 ],
              [ 1, 1, 1, 1, 0, 1 ],
              [ 1, 1, 1, 1, 1, 0 ] ];

    PrintMatrix(Adj02)

    # Compute rank of adj02 ...

    adj02rank = matrix_rank(Adj02)
    print("--- Rank = {:6,.1f}".format( adj02rank ));

    # Compute rank of Adj02 + I ...

    adj02I = numpy.matlib.identity(6,dtype=int);
    print(Adj02+ adj02I)

    adj02rank = matrix_rank(Adj02 + adj02I )
    print("--- Rank(A+I) = {:6,.1f}".format( adj02rank ));

    # Compute eigenvalues of A ...

    w, v = LA.eig( Adj02 )

    print("--- Eigenvalues (A) ...");
    PrintVector(w)
    print("--- Eigenvectors (A) ...");
    PrintMatrix(v)

    # Compute eigenvalues of A + adj01I ...

    w, v = LA.eig( Adj02 + adj02I )

    print("--- Eigenvalues (A+I) ...");
    PrintVector(w)
    print("--- Eigenvectors (A+I) ...");
    print(v)

    # Part 3: Six node line graph ...
    # -------------------------------------------

    print("--- ");
    print("--- Part 3: Six node line graph ... ");
    print("--- ----------------------------------------------------- ... ");
    print("--- Adjacency matrix Adj03 ... ");

    Adj03 = [ [ 0, 1, 0, 0, 0, 0 ],
              [ 1, 0, 1, 0, 0, 0 ],
              [ 0, 1, 0, 1, 0, 0 ],
              [ 0, 0, 1, 0, 1, 0 ],
              [ 0, 0, 0, 1, 0, 1 ],
              [ 0, 0, 0, 0, 1, 0 ] ];

    PrintMatrix(Adj03)

    # Compute rank of Adj03 ...

    adj03rank = matrix_rank(Adj03)
    print("--- Rank = {:6,.1f}".format( adj03rank ));

    # Compute rank of adj03 + I ...

    adj03I = numpy.matlib.identity(6,dtype=int);

    print(Adj03+ adj03I)

    adj03rank = matrix_rank(Adj03 + adj03I )
    print("--- Rank(A+I) = {:6,.1f}".format( adj03rank ));

    # Compute eigenvalues of A ...

    w, v = LA.eig( Adj03 )

    print("--- Eigenvalues (A) ...");
    PrintVector(w)
    print("--- Eigenvectors (A) ...");
    PrintMatrix(v)

    # Compute eigenvalues of A + adj01I ...

    w, v = LA.eig( Adj03 + adj03I )

    print("--- Eigenvalues (A+I) ...");
    PrintVector(w)
    print("--- Eigenvectors (A+I) ...");
    print(v)

    print("--- ======================================== ... ");
    print("--- Leave TestNumpySVD.main()                ... ");

# call the main method ...

main()
