# ===============================================================
# TestKeras-XNOR-XOR-Problem.py: Use kerase to solve XNOR, XOR
# gate problem.
# 
# Required Behavior: 
# 
# ------------------------------------------
#    Input 1     Input 2  -->  XNOR      XOR
# ------------------------------------------
#          0           0          1        0
#          0           1          0        1
#          1           0          0        1
#          1           1          1        0
# ------------------------------------------
# 
# Network Architecture:
# --------------------------------------------------------------- 
# 2 input nodes, 2 nodes on hidden layer, 2 nodes on output. 
# 
# Weighted edges: 
# 
# input  layer --> hidden layer: connected by 2x2 = 4 weights. 
# hidden layer --> output layer: connected by 2x2 = 4 weights. 
# 
# Written by: Mark Austin                           November 2020 
# ===============================================================

import numpy as np
from tensorflow import keras
from keras.models import Sequential
from keras.layers.core import Dense, Activation
from keras.optimizers import SGD

# main method ...

def main():
    print("--- Enter TestKeras-XNOR-XOR-Problem.main()  ... ");
    print("--- ======================================== ... ");

    print("--- Training data ...")

    training_data = np.array([[0,0],[0,1],[1,0],[1,1]], "float32")
    print(training_data)

    print("--- Target data for XNOR, XOR problem ...")

    target_data = np.array([[1,0],[0,1],[0,1],[1,0]], "float32")
    print(target_data)

    print("--- Define dense layers ...")

    layer1 = Dense(4, input_dim =2, activation = 'sigmoid')
    layer2 = Dense(2, activation = 'sigmoid' )

    print("--- Assemble Sequential model ...")

    model = Sequential()
    model.add( layer1 )
    model.add( layer2 )

    print("--- Compile model ...")

    model.compile(loss='mean_squared_error', optimizer='adam', metrics=['binary_accuracy'])

    print("--- Train model to fit data ...")

    model.fit( training_data, target_data, epochs=20000, verbose=2)

    # Retrieve and print layer weights and bias values ...

    np.set_printoptions(formatter={'float': '{: 0.5f}'.format})

    print("--- Layer 1: weights ...")

    print( layer1.get_weights()[0] )

    print("--- Layer 1: bias values ...")

    print( layer1.get_weights()[1] )

    print("--- Layer 2: weights ...")

    print ( layer2.get_weights()[0] )

    print("--- Layer 2: bias values ...")

    print ( layer2.get_weights()[1] )

    print("--- Trained model predictions ...")

    print( model.predict(training_data) )

    print("--- Rounded model predictions ...")

    print( model.predict(training_data).round())

    print("--- ======================================== ... ");
    print("--- Finished!! ... ");

# call the main method ...

main()
