# ===============================================================
# TestMatrice.py: Demonstrate various ways to create matrices and
# use them in Tensorflow ...
#  
# Written by: Mark Austin                              April 2021 
# ===============================================================

import numpy as np
import tensorflow as tf
from tensorflow.python.framework import ops

# main method ...

def main():
    print("--- Enter TestMatrices.main()                ... ");
    print("--- ======================================== ... ");

    # The main diagonal.

    print("--- Main diagonal ... ");

    diagonal = np.array([[1, 2, 3, 4],            # Input shape: (2, 4)
                         [5, 6, 7, 8]])
    print(diagonal)

    print("--- Craft identidy matrix from diagonal ... ");

    identity_matrix = tf.linalg.diag( diagonal )
    print(identity_matrix)

    # Identity matrix

    print("--- Identity matrix ... ");

    identity_matrix = tf.linalg.diag( [1.0,1.0,1.0] )
    print(identity_matrix)

    # 2x3 random norm matrix

    print("--- 2x3 matrix of normally distributed random nos ... ");

    A = tf.random.normal([2,3])
    print(A)

    # 2x3 constant matrix

    print("--- 2x3 constant matrix ... ");

    B = tf.fill([2,3], 5.0)
    print(B)

    # 3x2 random uniform matrix

    print("--- 2x3 matrix of uniformly distributed random nos ... ");

    C = tf.random.uniform([3,2])
    print(C)

    # Create matrix from np array

    print("--- Create matrix from np array ... ");

    D = tf.convert_to_tensor(np.array([[1., 2., 3.], [-3., -7., -1.], [0., 5., -2.]]))
    print(D)

    # Matrix addition/subtraction

    print("--- Matrix addition and subtraction ... ");

    # print ( tf.matadd(A,B) )

    # Matrix Multiplication

    print("--- Matrix multiplication ... ");

    # print( tf.matmul(B, identity_matrix) )

    # Matrix Transpose

    print("--- Matrix transpose ... ");

    print( tf.transpose(C) ) 

    # Matrix Determinant

    print("--- Matrix determinant ... ");

    print( tf.linalg.det(D) )

    # Matrix Inverse

    print("--- Matrix inverse ... ");

    print( tf.linalg.inv(D) )

    # Cholesky Decomposition

    print("--- Cholesky decomposition ... ");

    print( tf.linalg.cholesky(identity_matrix) )

    # Eigenvalues and Eigenvectors

    print("--- Eigenvalues and eigenvectors ... ");

    # print( tf.self_adjoint_eig(D) )

    print("--- ======================================== ... ");
    print("--- Finished!! ... ");

# call the main method ...

main()
