# ===============================================================
# TestTensorFlow-OR-Gate01.py: Create model for boolean OR gate
# in TensorFlow 2. 
# 
# Written by: Mark Austin                           November 2020 
# ===============================================================

import numpy as np
import tensorflow as tf
from tensorflow import keras

# ================================================================
# Create model for boolean OR gate ...
# 
# Note: Initialize the weights to `2.0` and the bias to `2.0`
#       In practice, these should be initialized to random values
#       self.W1=tf.random.normal([1])
# 
# This section of code by: Sumeet Suman
# ================================================================

class Model(object):
 
   def __init__(self):
     self.W1 = tf.Variable(2.)
     self.W2 = tf.Variable(2.)
     self.b  = tf.Variable(2.)
   
   def __call__(self, x1,x2):
     self.K1=self.W1 * x1 + self.W2 * x2 + self.b
     self.Output = tf.keras.activations.sigmoid(self.K1)
     return(self.Output)

# Compute cost ...

def compute_cost(target_y, predicted_y):
    c=tf.keras.losses.BinaryCrossentropy()
    return(c(target_y,predicted_y))

def None_to_Zero(v):
    if v==None:
       v=0
    return v

# Train model for boolean or gate ...

def train(model, X1, X2, Y2, learning_rate):
    with tf.GradientTape() as t:
            current_loss = compute_cost(Y2,model(X1,X2))
    dW1,dW2,db=t.gradient(current_loss,[model.W1,model.W2,model.b])          
    dW1 = None_to_Zero(dW1)
    db  = None_to_Zero(db)
    dW2 = None_to_Zero(dW2)
    model.W1.assign_sub(learning_rate * dW1)
    model.b.assign_sub( learning_rate * db)
    model.W2.assign_sub(learning_rate * dW2)

# main method ...

def main():
    print("--- Enter TestTensorFlow-OR-Gate01.main()    ... ");
    print("--- ======================================== ... ");

    # Initialize training data  ...

    print("--- Part 1: Initialize training data ... ");

    x1   = tf.Variable(initial_value=[0.0,1.0,0.0,1.0])
    x2   = tf.Variable(initial_value=[0.0,0.0,1.0,1.0])
    y_or = tf.Variable(initial_value=[0.0,1.0,1.0,1.0])

    # Create model ...

    print("--- Part 2: Create model ... ");

    model=Model()

    # Train Boolean OR Gate

    print("--- Part 3: Train boolean OR Gate ... ");

    epochs=range(10000)
    learning_rate = 0.1
    for epoch in epochs:
       train(model, x1, x2, y_or, learning_rate )

    # Retrieve details of trained model ...

    print("--- Part 4: Retrieve details of trained model ... ");

    w_or = {'W1':model.W1.numpy(),'W2':model.W2.numpy(),'b':model.b.numpy() }
    print( w_or )

    # Verify and print results ...

    print("--- Part 5: Verify results ... ");

    print( model(x1, x2).numpy() )
    print( np.round( model(x1, x2).numpy() ) )

    print("--- ======================================== ... ");
    print("--- Finished!! ... ");

# call the main method ...

main()
