# ================================================================
# TestNeuralNetwork02.py: Case Study: Predicting Virus contraction
# with a neural net.
# 
# -------------------------------------------------------
#            Loss of   Weight    Runny    Body
#    Person    Smell     Loss     Nose    Pain  Positive?
# -------------------------------------------------------
#         1        1        0        0       1          1
#         2        1        0        0       0          1
#         3        0        0        1       1          0
#         4        0        1        0       0          0
#         5        1        1        0       0          1
#         6        0        0        1       1          1
#         7        0        0        0       1          0
#         8        0        0        1       0          0
# -------------------------------------------------------
# 
# Note: input [0,0,1,1] --> 0 or 1. 
# 
# Reference: Shukla, et al., Neural Networks from Scratch with
# Python Code and Math in Detail, Towards AI, 2020  
# 
# Modified by: Mark Austin                           October, 2020 
# ================================================================

import math
import matplotlib
import matplotlib.pyplot as plt
import numpy as np

# Define Sigmoid function:

def sigmoid(x):
   return 1/(1+np.exp(-x))

# Define derivative of Sigmoid function:

def sigmoid_der(x):
   return sigmoid(x)*(1-sigmoid(x))

# main method ...

def main():
    print("--- Enter TestNeuralNetwork01.main()      ... ");
    print("--- ===================================== ... ");

    print("--- ");
    print("--- Part 1: Define input features ... ");

    input_features = np.array( [ [1,0,0,1],
                                 [1,0,0,0],
                                 [0,0,1,1],
                                 [0,1,0,0],
                                 [1,1,0,0],
                                 [0,0,1,1],
                                 [0,0,0,1],
                                 [0,0,1,0] ] )

    print (input_features.shape)
    print (input_features)

    # Define target output:

    print("--- ");
    print("--- Part 2: Define target output ... ");

    target_output = np.array([[1,1,0,0,1,1,0,0]])

    # Reshaping target output into vector:

    target_output = target_output.reshape(8,1)
    print (target_output)

    print("--- ");
    print("--- Part 3: Initial weights and learning parameters ... ");

    weights = np.array([ [0.1],
                         [0.2],
                         [0.3],
                         [0.4] ])
    print(weights.shape)
    print (weights) 

    bias = 0.3       # Bias weight: 
    lr   = 0.10      # Learning Rate:

    # ========================================================
    # Main logic for neural network:
    # ========================================================

    print("--- ");
    print("--- Part 4: Train Neural Network ... ");
    print("--- ---------------------------- ... ");
    print("--- ");

    # Running our code 20000 times:

    for epoch in range(20000):

       print ("--- Start epoch: %f ..." %(epoch))

       # feedforward input, feedforward output, back propogation ...

       print ("---   Step 1: Compute: feedforward input, feedforward output ...")

       inputs = input_features                 
       in_o   = np.dot(inputs, weights) + bias 
       out_o  = sigmoid(in_o)                 

       # Calculate error in computed output ...

       print("---   Step 2: Calculate error in computed output ...")

       error = out_o - target_output

       # Calculate derivative:

       print("---   Step 3: Calculate derivative ...")

       derror_douto = error
       douto_dino = sigmoid_der(out_o)

       # Multiplying individual derivatives:

       deriv = derror_douto * douto_dino 

       # Finding the transpose of input_features:

       inputs = input_features.T
       deriv_final = np.dot(inputs,deriv)

       # Update the weights values:

       print("---   Step 4: Update values of weights and bias weight ...")

       weights -= lr * deriv_final
       for i in deriv:
           bias -= lr * i #Check the final values for weight and biasprint (weights)

       # Summary of iteration results ...

       print("---   Step 5: Summary of iteration results ... ");
       print("---   Weights:");
       print(weights) 
       print("---   bias: %f ... \n" %(bias))

    # Print summary of results ...

    print("--- ");
    print("--- Part 5: Summary of weights and biases ... ");
    print("--- ------------------------------------- ... ");
    print("--- Weights:");

    print (weights) 

    print("--- Bias: %f ... \n" %(bias))

    print("--- ");
    print("--- Part 6: Use trained network to make predictions ... ");
    print("--- ----------------------------------------------- ... ");

    print("--- ");
    print("--- Part 6.1: Verify input [1,0,0,1] --> 1 ... ");
    print("--- ");

    single_point = np.array([1,0,0,1]) #1st step:
    result1 = np.dot(single_point, weights) + bias #2nd step:
    result2 = sigmoid(result1) #Print final result

    print ("--- Single input point [1,0] ...")
    print ("--- Result 1: %f ..." %(result1))
    print ("--- Result 2: %f ..." %(result2))

    print("--- ");
    print("--- Part 6.2: Verify input [1,0,0,0] --> 1 ... ");
    print("--- ");

    single_point = np.array([1,0,0,0]) #1st step:
    result1 = np.dot(single_point, weights) + bias #2nd step:
    result2 = sigmoid(result1) #Print final result

    print ("--- Single input point [1,0,0,0] ...")
    print ("--- Result 1: %f ..." %(result1))
    print ("--- Result 2: %f ..." %(result2))

    print("--- ");
    print("--- Part 6.3: Verify input [0,0,1,1] --> [0,1] ... ");
    print("--- ");

    single_point = np.array([0,0,1,1]) #1st step:
    result1 = np.dot(single_point, weights) + bias #2nd step:
    result2 = sigmoid(result1) #Print final result

    print ("--- Single input point [0,0,1,1] ...")
    print ("--- Result 1: %f ..." %(result1))
    print ("--- Result 2: %f ..." %(result2))

    print("--- ");
    print("--- Part 6.4: Verify input [0,0,0,1] --> 0 ... ");
    print("--- ");

    single_point = np.array([0,0,0,1]) #1st step:
    result1 = np.dot(single_point, weights) + bias #2nd step:
    result2 = sigmoid(result1) #Print final result

    print ("--- Single input point [0,0,0,1] ...")
    print ("--- Result 1: %f ..." %(result1))
    print ("--- Result 2: %f ..." %(result2))

    print("--- ===================================== ... ");
    print("--- Finished TestNeuralNetwork01.main()   ... ");

# call the main method ...

main()
